'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var rxjs = require('rxjs');
var operators = require('rxjs/operators');
var utils = require('../../common/utils.js');

const defaultDeserializer = message => message.data;

class RxWebSocket {
  #websocket;
  #opened$;
  #message$;
  #events$ = new rxjs.Subject();

  constructor(url, {
    protocols,
    clientOptions,
    deserializer = defaultDeserializer,
    debug = false
  }) {
    const websocket = utils.isBrowser() ? new WebSocket(url, protocols) : new (require('ws'))(url, protocols, clientOptions);
    const close$ = rxjs.fromEvent(websocket, 'close').pipe(rxjs.take(1), operators.share());
    const open$ = rxjs.fromEvent(websocket, 'open').pipe(operators.takeUntil(close$), rxjs.shareReplay(1), rxjs.take(1));
    const message$ = rxjs.fromEvent(websocket, 'message').pipe(operators.takeUntil(close$), rxjs.shareReplay(1));
    const error$ = rxjs.fromEvent(websocket, 'error').pipe(operators.takeUntil(close$), rxjs.mergeMap(rxjs.throwError));
    rxjs.merge(open$, message$, error$, close$).subscribe(this.#events$);
    this.#websocket = websocket;
    this.#opened$ = open$;
    this.#message$ = message$.pipe(operators.map(deserializer));

    if (debug) {
      this.#events$.subscribe(ev => console.log(`[websocket] event: ${ev.type},`, ev === null || ev === void 0 ? void 0 : ev.data));
      this.#websocket.send = new Proxy(this.#websocket.send, {
        apply: (target, thisArg, args) => {
          console.log('[websocket] send:', args[0]);
          return target.call(this.#websocket, args[0]);
        }
      });
    }
  }

  get active() {
    return !this.#events$.closed;
  }

  get message() {
    return this.#message$;
  }

  send(data) {
    return this.#opened$.pipe(rxjs.mergeMap(() => {
      this.#websocket.send(data);
      return this.message;
    }));
  }

}

exports.RxWebSocket = RxWebSocket;
//# sourceMappingURL=websocket.js.map
