import { Subject, fromEvent, take, shareReplay, mergeMap, throwError, merge } from 'rxjs';
import { share, takeUntil, map } from 'rxjs/operators';
import { isBrowser } from '../../common/utils.js';

const defaultDeserializer = (message) => message.data;
class RxWebSocket {
    #websocket;
    #opened$;
    #message$;
    #events$ = new Subject();
    constructor(url, { protocols, clientOptions, deserializer = defaultDeserializer, debug = false }) {
        const websocket = isBrowser() ? new WebSocket(url, protocols) : new (require('ws'))(url, protocols, clientOptions);
        const close$ = fromEvent(websocket, 'close').pipe(take(1), share());
        const open$ = fromEvent(websocket, 'open').pipe(takeUntil(close$), shareReplay(1), take(1));
        const message$ = fromEvent(websocket, 'message').pipe(takeUntil(close$), shareReplay(1));
        const error$ = fromEvent(websocket, 'error').pipe(takeUntil(close$), mergeMap(throwError));
        merge(open$, message$, error$, close$).subscribe(this.#events$);
        this.#websocket = websocket;
        this.#opened$ = open$;
        this.#message$ = message$.pipe(map(deserializer));
        if (debug) {
            this.#events$.subscribe((ev) => console.log(`[websocket] event: ${ev.type},`, ev?.data));
            this.#websocket.send = new Proxy(this.#websocket.send, {
                apply: (target, thisArg, args) => {
                    console.log('[websocket] send:', args[0]);
                    return target.call(this.#websocket, args[0]);
                },
            });
        }
    }
    get active() {
        return !this.#events$.closed;
    }
    get message() {
        return this.#message$;
    }
    send(data) {
        return this.#opened$.pipe(mergeMap(() => {
            this.#websocket.send(data);
            return this.message;
        }));
    }
}

export { RxWebSocket };
//# sourceMappingURL=websocket.js.map
