import { BehaviorSubject, firstValueFrom, mergeMap, interval } from 'rxjs';
import { first, timeout, takeUntil } from 'rxjs/operators';
import { constants } from '../common/constants.js';
import { toSocketIoMessagePayload, isResponseMessageOf, scrapboxDeserializer, isConnectionMessage, isResponseOf } from './internal/message.js';
import { createChanges } from './internal/request.js';
import { isNotNull, getAuthCookieValue } from './internal/utils.js';
import { RxWebSocket } from './internal/websocket.js';

class ScrapboxWebsocketHandler {
    #socket$ = new BehaviorSubject(null);
    #room = null;
    #sid = 0;
    #options;
    constructor(options) {
        this.#options = options;
    }
    async commit({ projectId, userId, pageId, parentId, changes, }) {
        await this.#join({ projectId, pageId });
        return this.#send({
            method: 'commit',
            data: { kind: 'page', userId, projectId, pageId, parentId, changes: createChanges(changes, userId), cursor: null, freeze: true },
        });
    }
    async #join({ projectId, pageId }) {
        // join room if needed
        if (this.#room?.projectId === projectId && this.#room?.pageId === pageId) {
            return;
        }
        await this.#send({
            method: 'room:join',
            data: { pageId, projectId, projectUpdatesStream: false },
        });
        this.#room = { projectId, pageId };
    }
    #send(data) {
        const sid = String(this.#sid++);
        return firstValueFrom(this.#getSocket().pipe(first(isNotNull), mergeMap((socket) => socket.send(toSocketIoMessagePayload(sid, data)).pipe(first(isResponseMessageOf(sid)), timeout(constants.websocket.responseTimeout)))));
    }
    #getSocket() {
        if (!this.#socket$.getValue()?.active) {
            return this.#openNewSocket();
        }
        return this.#socket$.asObservable();
    }
    #openNewSocket() {
        const socket = new RxWebSocket(constants.websocket.endpoint, {
            clientOptions: { headers: { Origin: constants.websocket.origin, Cookie: getAuthCookieValue(this.#options.token ?? '') } },
            deserializer: scrapboxDeserializer,
            debug: this.#options.debug,
        });
        return socket.message.pipe(first(isConnectionMessage), mergeMap(([, data]) => {
            return socket.send(constants.websocket.packetTypes.connected).pipe(first(isResponseOf(constants.websocket.packetTypes.connected)), mergeMap(() => {
                interval(data.pingInterval)
                    .pipe(takeUntil(socket.message))
                    .subscribe(() => socket.send(constants.websocket.packetTypes.ping));
                this.#socket$.next(socket);
                return this.#socket$.asObservable();
            }));
        }));
    }
}

export { ScrapboxWebsocketHandler };
//# sourceMappingURL=websocket-handler.js.map
